#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Extracts meta-data from MD trajectories. This tool comes with an powerful interpreter of reverse polish notation to represent complex system properties in a condensed way.

Reverse polish notation
-----------------------
A compact notation to express arbitrary mathematical operations while refraining from the extensive usage of braces is called the reverse polish notation. In general, it is a postfix syntax, that is, the operator is written after the operands. For example the first line becomes

.. code:: sh 

	(3 + 4) * (5 + 6) / 7
	3 4 + 5 6 + * 7 /

The list of the operands waiting to be consumed by an operator is called stack. Objects can be placed on the stack arbitrarily. However, this implementation requires you to end with a single entry on the stack. Each of the objects can be both a scalar and a vector.

Examples
--------

Suppose you have a molecule with three points of interest, which shall be called A, B, and C. You are interested in the angle of the vectors BA and CB with the z-axis and the angle between these two vectors. Then the following lines will give you three angles in radians per frame.

.. code:: sh

	f_timeseries.py input.psf input.dcd --periodic  \\
		--cmd 'selection A:com; selection B:com;diff;0,0,1;angle' \\
		--cmd 'selection B:com; selection C:com;diff;0,0,1;angle' \\
		--cmd 'selection A:com; selection B:com;diff;selection B:com; selection C:com;diff;angle'

Shortcuts
---------
Sometimes a command will be used frequently. Hence, there are shortcuts in this case. The following shortcuts are available. Each time, the two lines are identical.

.. code:: sh

	# prints the side lengths (x, y, z) of the unit cell
	box
	boxdimen;1,1,1,0,0,0;mask;3;pile

Supported formats
-----------------

All file formats of :ref:`MDAnalysis <supported coordinate formats>` are supported. Input files may be compressed using gzip or bzip2.

.. note:: File formats are recognized by file extension. Therefore, you have to stick to the standard file extensions.

.. warning:: Only rectangular simulation boxes are currently supported.

.. todo:: Guess input file format.

Command line interface
----------------------

.. program:: f_timeseries.py

.. option:: topology

   The topology file to work with. 

.. option:: coordinates

   The coordinate file to work with.

.. option:: --frames

   The zero-based frame selection that allows for slicing. The general syntax is 'start:stop:step' which would select all frames from `start` to `stop` (excluding `stop`) while skipping `step` frames inbetween. Selecting single frames by specifying a single number is possible. Negative indices count backwards from the end of the trajectory.

.. option:: --periodic

   Whether to wrap atoms for spatial selection criteria back to the original cell.

.. option:: --indexfile

   Files that contain named groups of atoms based on their indices. Files have to be in GROMACS ndx-Format. Can be used multiple times.

.. option:: --cmd

   String concatenating the operators using reverse polish notation together with atom selectors. This command can be issued multiple times on the command line. Every option will result in an additional entry printed in the resulting line per frame. The general notation is

   .. code:: sh 

   		selection:cmd;selection2:cmd2

   There are some operators that do not support selections. They are meant to be used this way

   .. code:: sh

   		:cmd;:cmd2

   You can put elements on the stack by yourself. They have to be numeric, but can be either a comma-separated vector or a scalar. The selection syntax is decribed separately. Please refer to the :ref:`section <atomselections>` on atom selections. The available operators are

   * `com` calculates the center of mass of the given selection. If the selection is empty, the center of mass of all atoms in the whole system is calculated. The result is a vector in cartesian coordinates. `com` does not consume any stack elements but rather adds another one.

   .. code:: sh

   		id 1-10:com

   * `dist` calculates the distance between two points in cartesian coordinates. The last two stack elements will be consumed and have to be vectors. The resulting scalar is put back on the stack. This operator does not support selections.

   .. code:: sh

   		1,1,1; 2,2,2; dist
   		id 1-10:com; id 11-20:com; dist

   * `diff` calculates the difference of the two last stack entries which only have to be of the same dimension. This operator does not support selections.

   .. code:: sh

   		3;4;diff
   		id 1-10:com; id 11-20:com; diff

   * `framenum` prints the number of the current frame in the trajectory. Selections are not supported. Please keep in mind that this frame number does not necessarily correspond with a simulation step and that the indices are zero-based.

   .. code:: sh

   		framenum

   * `boxdimen` puts the system box dimensions on the stack. The unit cell is defined by six elements: the length of the x, y, and z vector and the angles inbetween (alpha, beta, and gamma).

   .. code:: sh

   		boxdimen

   * `mask` discards some of the elements on the stack. The last element of the stack has to be a vector which is to be aligned with the end of the stack. Only stack elements with a non-zero mask entry are kept. For example, if you are intested in the x and y axis of the box dimensions only, you may use:

   .. code:: sh

   		boxdimen;1,1,0,0,0,0;mask

   * `prod` calculates the product of the last two stack elements.
   * `frac` calculated the quotient of the last two stack elements. Please note that the last stack element is the numerator.
   * `sum` adds the last two stack elements.
   * `flatten` takes a single vector from the stack and pushes its elements back.
   * `charge` calculates the charge density profile along a given vector. This command calculates the charge density of all molecules that do not belong to a residue named UNK from 0 to 60 Angstroms along the 0,0,1 vector using 100 bins. The result is pushed onto the stack as a vector that has as many elements as bins were used. 

   .. code:: sh

   		0,60;100;0,0,1;not resname UNK:charge

   * `fitplane` fits a plane to the atom coordinates of the given selection. The normalized normal vector of the plane is pushed onto the stack.

   .. code:: sh

   		type PL:fitplane

   .. note:: 

   		The algorithm used is Singular Value Decomposition. Please make sure that you only have only atoms for one plane as multiple parallel planes in the underlying structure will lead to spurious errors.

   * `pile` takes the last element on the stack and inteprets it as number of elements to transform into a vector. The resulting vector is pushed onto the stack. The following would print the simulation cell dimensions.

   .. code:: sh

   		boxdimen;1,1,1,0,0,0;mask;3;pile


Internal functions
------------------
"""

# system modules
import argparse
import math

# custom modules
import f_core as fc
import output_tracker as ot

# third-party modules
import numpy as np

# command line parsing
parser = argparse.ArgumentParser(description='Calculates collective properties from MD trajectories.')
parser.add_argument('topology', type=str, help='The topology to work with.')
parser.add_argument('coordinates', type=str, help='The coordinates to work with.')
parser.add_argument('--frames', type=str, help='Frame selection.', default='')
parser.add_argument('--periodic', help='Whether to wrap atoms for spatial selection criteria back to the original cell.', action='store_true')
parser.add_argument('--cmd', type=str, help='Command that will produce output.', action='append')
parser.add_argument('--indexfile', type=str, nargs='+', help='GROMACS style index file for named groups.')

def _operator_com(selection, universe, output):
	"""
	Reverse polish notation operator that calculates the center of mass.

	Parameters
	----------
	selection : String
		The CHARMM-style selection request. May contain atom ids, as well.
	universe : :class:`mda:MDAnalysis.core.AtomGroup.Universe` 
		The universe which trajectory is to be analyzed.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	numpy.array 
		Center of mass in cartesian coordinates. Length unit is Angstrom.
	"""
	ag = fc.mda_select_id(universe, selection, output)

	if len(ag) == 0:
		output.print_error(fc.messages.EMPTY_SELECTION)

	return ag.centerOfMass()

def _dump_stack(stack, output):
	"""
	Lists the current stack elements.

	Parameters
	----------
	stack : iterable
		The current stack.
	output : :class:`output_tracker` instance
	  Error reporting.	
	"""
	output.print_info('Stack elements:').add_level()
	for s in stack:
		output.print_info('%s' % str(s))
	output.del_level()

def _require_stack_length(stack, length, output):
	"""
	Takes the requested number of elements from the stack.

	Parameters
	----------
	stack : iterable
		The current stack.
	length : int
		The number of elements to take from the end of the stack.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	tuple
		Tuple of length two with iterables: the new stack and the popped entries.
	"""
	if len(stack) < length:
		_dump_stack(stack, output)
		output.print_info('Stack size %d' % len(stack))
		output.print_error(fc.messages.STACK_POP)

	return (stack[:-length], stack[-length:])

def _operator_dist(stack, output):
	"""
	Reverse polish notation operator that calculates the distance between the last two stack elements.

	Parameters
	----------
	stack : iterable
		The current stack.
	output : :class:`output_tracker` instance
	  Error reporting.	

	Returns
	-------
	iterable
		The new stack.
	"""
	stock, arguments = _require_stack_length(stack, 2, output)
	arg1, arg2 = arguments

	if len(arg1) != 3 or len(arg2) != 3:
		output.print_error(fc.messages.STACK_DIMENSION)

	dist = math.sqrt(sum([(a-b)**2 for a, b in zip(arg1, arg2)]))

	stock.append(dist)
	return stock

def _operator_diff(stack, output):
	"""
	Reverse polish notation operator that calculates the difference between the last two stack elements.

	Parameters
	----------
	stack : iterable
		The current stack.
	output : :class:`output_tracker` instance
	  Error reporting.	

	Returns
	-------
	iterable
		The new stack.
	"""	
	stock, arguments = _require_stack_length(stack, 2, output)
	arg1, arg2 = arguments

	if len(arg1) != len(arg2):
		output.print_error(fc.messages.DIM_MISMATCH)

	diff = [a-b for a, b in zip(arg1, arg2)]

	if len(diff) == 1:
		stock.append(diff[0])
	else:
		stock.append(diff)
	return stock

def _operator_angle(stack, output):
	"""
	Reverse polish notation operator that calculates the angle between the last two stack elements.

	The last two stack elements have to be a vector.

	Parameters
	----------
	stack : iterable
		The current stack.
	output : :class:`output_tracker` instance
	  Error reporting.	

	Returns
	-------
	iterable
		The new stack.
	"""
	stock, arguments = _require_stack_length(stack, 2, output)
	arg1, arg2 = arguments

	if len(arg1) != 3 or len(arg2) != 3:
		output.print_error(fc.messages.STACK_DIMENSION)

	phi = fc.angle(arg1, arg2, output)
	stock.append(phi)
	return stock

def _operator_boxdimen(stack, universe, framenum, output):
	"""
	Reverse polish notation operator that puts the simulation box dimensions on the stack.

	Parameters
	----------
	stack : iterable
		The current stack.
	universe : :class:`mda:MDAnalysis.core.AtomGroup.Universe` 
		The universe which trajectory is to be analyzed.
	framenum : Integer
		Current frame number. One-based.
	output : :class:`output_tracker` instance
	  Error reporting.
	"""
	stack += list(universe.trajectory[framenum-1].dimensions)
	return stack

def _operator_mask(stack, output):
	"""
	Reverse polish notation operator that filters amongst the last stack elements.

	Parameters
	----------
	stack : iterable
		The current stack.
	output : :class:`output_tracker` instance
	  Error reporting.	

	Returns
	-------
	iterable
		The new stack.
	"""
	stock, mask = _require_stack_length(stack, 1, output)
	
	l = len(mask[0])
	stock, arguments = _require_stack_length(stock, l, output)
	keep = [val for mask, val in zip(mask[0], arguments) if mask != 0]
	stock += keep
	return stock

def _operator_prod(stack, output):
	"""
	Reverse polish notation operator that calculates the product of the last two stack elements.

	Parameters
	----------
	stack : iterable
		The current stack.
	output : :class:`output_tracker` instance
	  Error reporting.	

	Returns
	-------
	iterable
		The new stack.	
	"""
	stock, arguments = _require_stack_length(stack, 2, output)
	arguments = map(np.array, arguments)

	try:
		res = arguments[0]*arguments[1]
	except:
		output.print_error(fc.messages.STACK_DIMENSION)

	stock.append(res)
	return stock

def _operator_sum(stack, output):
	"""
	Reverse polish notation operator that calculates the sum of the last two stack elements.

	Parameters
	----------
	stack : iterable
		The current stack.
	output : :class:`output_tracker` instance
	  Error reporting.	

	Returns
	-------
	iterable
		The new stack.	
	"""
	stock, arguments = _require_stack_length(stack, 2, output)
	arguments = map(np.array, arguments)

	try:
		res = arguments[0]+arguments[1]
	except:
		output.print_error(fc.messages.STACK_DIMENSION)

	stock.append(res)
	return stock

def _operator_frac(stack, output):
	"""
	Reverse polish notation operator that calculates the quotient of the last two stack elements.

	Note that the last stack element is treated as numerator.

	Parameters
	----------
	stack : iterable
		The current stack.
	output : :class:`output_tracker` instance
	  Error reporting.	

	Returns
	-------
	iterable
		The new stack.	
	"""
	stock, arguments = _require_stack_length(stack, 2, output)
	arguments = map(np.array, arguments)

	try:
		res = arguments[1]/arguments[0]
	except:
		output.print_error(fc.messages.STACK_DIMENSION)

	stock.append(res)
	return stock

def _operator_charge(stack, selection, universe, framenum, output):
	"""
	Reverse polish notation operator that calculates the charge density profile.	

	.. note:: 
		Only works correctly for charge density profiles along the z axis.

	.. todo::
		Implement vector choice.

	Parameters
	----------
	stack : iterable
		The current stack.
	selection : String
		The CHARMM-style selection request. May contain atom ids, as well.
	universe : :class:`mda:MDAnalysis.core.AtomGroup.Universe` 
		The universe which trajectory is to be analyzed.
	framenum : Integer
		Current frame number. One-based.
	output : :class:`output_tracker` instance
	  Error reporting.	

	Returns
	-------
	iterable
		The new stack.	
	"""
	stock, arguments = _require_stack_length(stack, 3, output)
	bounds, numbins, projection = arguments
	numbins = int(numbins[0])
	minval = min(bounds)
	maxval = max(bounds)
	projection = np.array(projection)

	if len(projection) != 3:
		output.print_error(fc.messages.STACK_DIMENSION)
	# TODO: test for null vector
	# TODO: more checks on input values

	ag = fc.mda_select_id(universe, selection, output)

	bindata = [0]*numbins
	binwidth = (maxval-minval)/numbins
	dimensions = universe.trajectory[framenum].dimensions

	# TODO: more general formula
	area = dimensions[0]*dimensions[1]
	for atom in ag:
		intvar = projection.dot(atom.pos)
		if intvar < minval or intvar > maxval:
			continue

		bin = int(min(intvar % numbins, numbins-1))
		bindata[bin] += abs(atom.charge)/(area*binwidth)

	stock.append(bindata)
	return stock

def _operator_fitplane(stack, selection, universe, framenum, output):
	"""
	Reverse polish notation operator that fits a plane to coordinates.

	Parameters
	----------
	stack : iterable
		The current stack.
	selection : String
		The CHARMM-style selection request. May contain atom ids, as well.
	universe : :class:`mda:MDAnalysis.core.AtomGroup.Universe` 
		The universe which trajectory is to be analyzed.
	framenum : Integer
		Current frame number. One-based.
	output : :class:`output_tracker` instance
	  Error reporting.	

	Returns
	-------
	iterable
		The new stack.	
	"""
	fc.mda_patch_parser()
	ag = fc.mda_select_id(universe, selection, output)
	if len(ag) == 0:
		output.print_error(fc.messages.EMPTY_SELECTION)
	if len(ag) < 3:
		output.print_info('Selected %d atoms.' % len(ag))
		output.print_error(fc.messages.UNDEF_PLANE)
	orig_pos = ag.get_positions(ts=universe.trajectory[framenum])
	pos = orig_pos - sum(orig_pos)/len(orig_pos)
	
	# calculate plane
	rows, cols = pos.shape
	p = np.ones((rows, 1))
	AB = np.hstack([pos, p])
	u, d, v = np.linalg.svd(AB, True)
	B = v[3,:]
	nn = np.linalg.norm(B[0:3])
	B = B / nn
	
	# output
	stack.append(B[0:3])
	return stack

def _operator_flatten(stack, output):
	"""
	Reverse polish notation operator that puts every element of an vector on the stack separately.

	Parameters
	----------
	stack : iterable
		The current stack.
	output : :class:`output_tracker` instance
		Error reporting.	

	Returns
	-------
	Iterable
		The new stack.		
	"""
	stock, arguments = _require_stack_length(stack, 1, output)

	try:
		l = len(arguments[0])
	except:
		output.print_error(fc.messages.STACK_DIMENSION)

	for e in arguments[0]:
		stock.append(e)

	return stock

def _operator_pile(stack, output):
	"""
	Reverse polish notation operator that joins elements on the stack info one single vector.

	Parameters
	----------
	stack : Iterable
		The current stack.
	output : :class:`output_tracker` instance
		Error reporting.

	Returns
	-------
	Iterable
		The new stack.
	"""

	stock, arguments = _require_stack_length(stack, 1, output)

	try:
		num_elements = int(arguments[0])
	except:
		output.print_info('Got %s.' % arguments[0])
		output.print_error(fc.messages.INT_CONVERSION)

	if num_elements < 0:
		output.print_info('Got %d.' % num_elements)
		output.print_error(fc.messages.NEGATIVE_LENGTH)

	stock, arguments = _require_stack_length(stock, num_elements, output)

	stock.append(np.array(arguments))

	return stock

def _execute(columns, command, framenum, universe, output):
	"""
	Reverse polish notation parser.

	Parameters
	----------
	columns : iterable
		Output of former :option:`--cmd` arguments.
	command : String
		Current request in reverse polish notation to execute.
	framenum : Integer
		Current frame number. One-based.
	universe : :class:`mda:MDAnalysis.core.AtomGroup.Universe` 
		The universe which trajectory is to be analyzed.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	iterable
		Results of all :option:`--cmd` requests processed so far.
	"""
	parts = [_.strip() for _ in command.split(';')]
	stack = []

	# shortcuts
	while 'box' in parts:
		idx = parts.index('box')
		parts = parts[0:idx] + 'boxdimen;1,1,1,0,0,0;mask;3;pile'.split(';') + parts[idx+1:]

	# commands
	for part in parts:
		temp = [_.strip() for _ in part.split(':')]
		if len(temp) == 1:
			if temp[0] == 'framenum':
				stack.append(framenum-1)
			elif temp[0] == 'dist':
				stack = _operator_dist(stack, output)
			elif temp[0] == 'diff':
				stack = _operator_diff(stack, output)
			elif temp[0] == 'angle':
				stack = _operator_angle(stack, output)
			elif temp[0] == 'prod':
				stack = _operator_prod(stack, output)
			elif temp[0] == 'sum':
				stack = _operator_sum(stack, output)
			elif temp[0] == 'frac':
				stack = _operator_frac(stack, output)
			elif temp[0] == 'flatten':
				stack = _operator_flatten(stack, output)
			elif temp[0] == 'pile':
				stack = _operator_pile(stack, output)
			elif temp[0] == 'mask':
				stack = _operator_mask(stack, output)
			elif temp[0] == 'boxdimen':
				stack = _operator_boxdimen(stack, universe, framenum, output)
			else:
				try:
					data = [float(_) for _ in temp[0].split(',')]
					if len(data) == 1 and ',' not in temp[0]:
						# scalar pushed onto the stack
						stack.append(data[0])
					else:
						# vector intended by the user
						stack.append(data)
				except:
					output.print_info('Got operator %s.' % temp[0])
					output.print_error(fc.messages.UNKNOWN_CMD)
		elif len(temp) == 2:
			selection, operator = temp
			if operator == 'com':
				stack.append(_operator_com(selection, universe, output))
			elif operator == 'charge':
				stack = _operator_charge(stack, selection, universe, framenum, output)
			elif operator == 'fitplane':
				stack = _operator_fitplane(stack, selection, universe, framenum, output)
			else:
				output.print_info('Got operator %s.' % operator)
				output.print_error(fc.messages.UNKNOWN_CMD)
		else:
			output.print_info('Got command %s' % command)
			output.print_error(fc.messages.TERNARY_CMD)

	if len(stack) != 1:
		_dump_stack(stack, output)
		output.print_info('Command:').add_level()
		output.print_info(command)
		output.del_level().print_error(fc.messages.NOT_CONSUMED)

	return columns + stack

def main():
	"""
	Main wrapper.
	"""
	# messages
	output = ot.output_tracker()
	messages = fc.messages

	# init
	mda = fc._require_mda(output)
	args = parser.parse_args()

	# read input files
	output.print_info('Reading source files.').add_level()
	output.print_info('Opening file %s.' % args.topology)
	output.print_info('Opening file %s.' % args.coordinates)

	# create Universe
	fc.mda_set_periodic(args.periodic, output)
	u = fc.mda_load_universe(args.topology, args.coordinates, output)
	output.print_info('Coordinate file holds %d frames.' % len(u.trajectory)).del_level()

	# parse selection
	f_sel = fc.parse_frame_selection(args.frames, output)
	groups = fc.parse_indexfiles(args.indexfile, output)
	u.add_index_groups(groups)

	# analyze trajectory
	try:
		l = len(args.cmd)
	except:
		output.print_error(messages.NO_CMD)
	for ts in u.trajectory[f_sel]:
		columns = []

		for cmd in args.cmd:
			columns = _execute(columns, cmd, u.trajectory.frame, u, output)

		for col in columns:
			try:
				l = len(col)
			except:
				print col,
				continue
			for e in col:
				print e,
		print

if __name__ == '__main__':
	main()
	exit(2)



# system modules
import math
import sys
import traceback

def colorexcept(exc_type, exc_value, exc_traceback): #isdoc
    frames = traceback.extract_tb(exc_traceback)
    framewidth = int(math.ceil(math.log(len(frames))/math.log(10)))
    filewidth = 0
    linewidth = 0
    functionwidth = 0
    for frame in frames:
        filewidth = max(filewidth, len(frame[0]))
        linewidth = max(linewidth, frame[1])
        functionwidth = max(functionwidth, len(frame[2]))
    linewidth = int(math.ceil(math.log(linewidth)/math.log(10)))
    print '\n\nTraceback: \033[1;92m{0}\033[0m: \033[1;91m{1}\033[0m'.format(exc_type.__name__, exc_value)
    for i in xrange(len(frames)):
        print ('\033[1;97m{0:' + str(framewidth) + '} \033[1;91m{1:' + str(filewidth) + '}\033[0m:{2:' + str(linewidth) + '} \033[1;92m{3:' + str(functionwidth) + '}\033[0m {4}').format(i, frames[i][0], frames[i][1], frames[i][2], frames[i][3])
    print 

if __name__ == 'colorexcept':
    if sys.stdout.isatty():
        sys.excepthook = colorexcept



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Extracts positional data from MD trajectories. The output can be filtered simultaneously by frame selections and CHARMM-style atom selections.

Examples
--------

Extracts frame 100, 200, and 300 from membrane.dcd.

.. code:: sh
   
   f_extract.py membrane.psf membrane.dcd output.dcd --frames 100:301:100

Extracts the complete trajectories of all atoms that are at most five Angstroms away from the origin in frame 10 (the eleventh frame) while wrapping atom coordinates periodically back to the simulation box.

.. code:: sh
  
   f_extract.py membrane.psf membrane.dcd output.dcd \\
		--charmm 'point 0.0 0.0 0.0 5.0' \\
		--selection_frame 10 \\
		--periodic

Extracts the last frame of a trajectory.

.. code:: sh

   f_extract.py membrane.psf membrane.dcd output.dcd --frames -1

Shows usage.

.. code:: sh

   f_extract.py -h

Supported formats
-----------------

All file formats of :ref:`MDAnalysis <supported coordinate formats>` are supported. Input files may be compressed using gzip or bzip2.

.. note:: File formats are recognized by file extension. Therefore, you have to stick to the standard file extensions.

.. note:: Some output formats may only support writing a single frame.

.. warning:: Only rectangular simulation boxes are currently supported.

Command line interface
----------------------

.. program:: f_extract.py

.. option:: topology

   The topology file to work with. 

.. option:: coordinates

   The coordinate file to work with.

.. option:: output

   Output file to write to.

.. option:: --frames

   The zero-based frame selection that allows for slicing. The general syntax is 'start:stop:step' which would select all frames from `start` to `stop` (excluding `stop`) while skipping `step` frames inbetween. Selecting single frames by specifying a single number is possible. Negative indices count backwards from the end of the trajectory.

.. option:: --charmm

   Please refer to the :ref:`section <atomselections>` on atom selections.

.. option:: --selection_frame

   Input frame to select on (zero-based). Only interesting for geometric criteria.

.. option:: --progress

   Progress report every time this many frames have been written.

.. option:: --force

   Whether to overwrite existing output files.

.. option:: --periodic

   Whether to wrap atoms for spatial selection criteria back to the original cell.

Open Issues
-----------

.. todo:: Single frame PDB output not functional.

.. todo:: Progress output: wrap last line.

.. todo:: Allow for comma-separated list of frames.

Internal functions
------------------
"""

# system modules
import argparse
import os
import os.path as path

# custom modules
import f_core as fc
import output_tracker as ot

# command line parsing
parser = argparse.ArgumentParser(description='Extracts positional data from MD trajectories.')
parser.add_argument('topology', type=str, help='The topology to work with.')
parser.add_argument('coordinates', type=str, help='The coordinates to work with.')
parser.add_argument('output', type=str, help='The coordinate file to write to.')
parser.add_argument('--frames', type=str, help='Frame selection.', default='all')
parser.add_argument('--charmm', type=str, help='CHARMM selection syntax.', default=None)
parser.add_argument('--selection_frame', type=int, help='Input frame to select on (zero-based).', default=0)
parser.add_argument('--progress', type=int, help='Progress report every time this many frames have been written.', default=100)
parser.add_argument('--force', help='Whether to overwrite existing files.', action='store_true')
parser.add_argument('--periodic', help='Whether to wrap atoms for spatial selection criteria back to the original cell.', action='store_true')

def main():
	"""
	Wrapper function.
	"""
	output = ot.output_tracker()
	mda = fc._require_mda(output)
	args = parser.parse_args()

	# messages
	messages = fc.messages

	# read input
	output.print_info('Reading source files.').add_level()
	output.print_info('Opening file %s.' % args.topology)
	output.print_info('Opening file %s.' % args.coordinates)

	# create Universe
	fc.mda_set_periodic(args.periodic, output)
	u = fc.mda_load_universe(args.topology, args.coordinates, output)
	output.print_info('Coordinate file holds %d frames.' % len(u.trajectory)).del_level()

	# parse frame selection
	f_sel = fc.parse_frame_selection(args.frames, output)

	# calculate number of frames
	numframes = fc.mda_get_framecount(universe, selection, output)
	if numframes is None:
		numframes = 0

	output.print_info('Starting extraction.').add_level()
	if not args.force and path.exists(args.output):
		output.print_error(messages.OUTPUT_EXISTS)

	curframe = 1
	if args.selection_frame < 0:
		args.selection_frame + len(u.trajectory)
	if args.selection_frame > len(u.trajectory):
		output.print_info('Selection frame is %d' % args.selection_frame)
		output.print_error(messages.INVALID_FRAME_SEL)

	initial_selection = fc.mda_select(u, args.charmm, output, args.selection_frame)
	output.print_info('Selected %d atoms.' % len(atom_nums))

	try:
		writer = mda.Writer(args.output, len(atom_nums))
	except Exception, e:
		output.print_info('MDAnalysis: %s' % str(e))
		output.print_error(messages.OUTPUT_FAILED)

	def print_stat(curframe, numframes, outfile, overwrite=True):
		statinfo = os.stat(outfile)
		output.print_info('Completed frame %8d/%8d. File size %6dMB.' % (curframe, numframes, statinfo.st_size/1024/2024), overwrite=overwrite)

	outfile = args.output
	for ts in u.trajectory[f_sel]:
		if curframe % args.progress == 0:
			print_stat(curframe, numframes, outfile, overwrite=True)
		curframe += 1
		writer.write(initial_selection.ts)
	writer.close()

	print_stat(numframes, numframes, outfile, overwrite=False)
	output.print_info('Finished extraction.')

if __name__ == '__main__':
	main()


# system modules
import math
import numpy
import numpy.linalg
import random
import fractions
import decimal
import collections

def vector_length(iterable):
    return math.sqrt(sum([_*_ for _ in iterable]))

def reverse_number(number):
    return int(str(number)[::-1])

def is_palindrome(number):
    return reverse_number(number) == number

def is_lychrel_number(candidate, max_iter=50):
    if max_iter < 1:
        raise ValueError('Invalid number of maximum iterations.')

    cur_iter = 1
    s = candidate
    while cur_iter <= max_iter:
        s = s + reverse_number(s)
        if is_palindrome(s):
            return (False, cur_iter)
        cur_iter += 1

    return (True, max_iter)

def digital_sum(number):
    return sum(map(int, str(number)))

def sqrt_2_series(iteration=1):
    if iteration < 0:
        raise ValueError('Invalid iteration count.')

    if iteration == 0:
        return fractions.Fraction(1, 1)

    base = fractions.Fraction(1, 2)
    while iteration > 1:
        iteration -= 1
        base = 1 / (2 + base)

    return 1 + base

def is_prime_sieve(num, sorted_primes=None):
    if num < 2:
        return False
    if num == 2:
        return True
    iterable = sorted_primes if sorted_primes is not None else xrange(2, int(math.ceil(math.sqrt(num)))+1)
    for i in iterable:
        if num % i == 0:
            return i
        if i*i > num:
            break
    return True

def digit_factorial_chain(num):
    yield num
    while True:
        digits = collections.Counter(str(num))
        num = 0
        for digit in digits:
            num += math.factorial(int(digit)) * digits[digit]
        yield num

def digit_square_chain(num):
    yield num
    while True:
        digits = collections.Counter(str(num))
        num = 0
        for digit in digits:
            num += int(digit)**2 * digits[digit]
        yield num

def is_prime_percentage(list_of_ints):
    return len([_ for _ in list_of_ints if is_prime_sieve(_)])*1.0/len(list_of_ints)

def anticlockwise_spiral_diagonals(side_length):
    if side_length < 3 or side_length % 2 == 0:
        return ValueError('Invalid side length.')

    steps = (side_length-1)/2

    result = [1]
    last = 1
    for step in range(1, steps+1):
        diff = 2*step
        result += [last + diff, last + 2*diff, last+3*diff, last+4*diff]
        last = result[-1]
    return result

def triangle(n):
    return n*(n+1)/2

def square(n):
    return n*n

def pentagonal(n):
    return n*(3*n-1)/2

def hexagonal(n):
    return n*(2*n-1)

def heptagonal(n):
    return n*(5*n-3)/2

def octagonal(n):
    return n*(3*n-2)

def sorted_number(n):
    return ''.join(sorted(str(n))[::-1])

def prime_factors_single(n): 
    pFact, limit, check, num = [], int(math.sqrt(n)) + 1, 2, n 
    if n == 1: return [1] 
    for check in range(2, limit): 
         while num % check == 0: 
            pFact.append(check) 
            num /= check 
    if num > 1: 
      pFact.append(num) 
    return pFact 

def prime_factors_sieve(n, sorted_primes):
    factors = []
    while True:
        candidate = is_prime_sieve(n, sorted_primes=sorted_primes)
        if candidate is True:
            factors.append(n)
            break
        else:
            factors.append(candidate)
            n /= candidate
    return factors

def prime_build_list(up_to):
    res = [2]
    i = 1
    up_to -= 2
    while i <= up_to:
        i += 2
        prime = True
        for p in res:
            if i % p == 0:
                prime = False
                break
            if p*p > i:
                break
        if prime:
            res.append(i)
    return res

def euler_totient(n, sorted_primes=None):
    factors = []
    if sorted_primes is None:
        factors = prime_factors_single(n)
    else:
        factors = prime_factors_sieve(n, sorted_primes)

    cnt = collections.Counter(factors)
    res = 1
    for val in cnt:
        res *= val**(cnt[val]-1)*(val-1)
    return res

def continued_fraction(n, precision=28):
    decimal.getcontext().prec = precision
    alpha = decimal.Decimal(n)

    while True:
        q, r = divmod(alpha, 1)
        yield (q, alpha)
        if r.is_zero():
            return

        alpha = 1/(alpha-q)

def detect_periodicity(iterable, limit=50, etol=0, valueqeualsstate=False):
    states = []
    values = []

    is_periodic = False
    periodic_state = None
    while len(values) <= limit:
        try:
            if valueqeualsstate:
                value = next(iterable)
                state = value
            else:
                value, state = next(iterable)
        except:
            break
        for former in states:
            if abs(former-state) <= etol:
                is_periodic = True
                periodic_state = former
        if is_periodic:
            break
        else:
            states.append(state)
            values.append(value)

    if is_periodic:
        split = states.index(periodic_state)
        return (is_periodic, values[:split], values[split:])
    else:
        return (is_periodic, values, [])

def all_indices(needle, haystack): #isdoc
    if not isinstance(haystack, list):
        raise ValueError('Expecting instance of list')

    indices = range(len(haystack))
    parts = zip(indices, haystack)
    return [x[0] for x in parts if x[1] == needle]

def list_diff(l, NullInclude=False): #isdoc
    if not isinstance(l, list):
        raise ValueError('Expecting instance of list')

    r = []
    if NullInclude:
        r.append(l[0])
    for i in range(len(l)-1):
        r.append(l[i+1]-l[i])
    return r

class vector(object): #isdoc #doc-contains: _size, _pos
    def __init__(self, size): #isdoc
        self._size = size
        self._pos = [0]*size

    def __str__(self): #isdoc
        s = 'vector['
        for i in range(self._size):
            s += '{0:10f} '.format(self._pos[i])
            if (i % 5) == 4:
                s += '\n         '
        return s + ']'

    def __mul__(self, factor): #isdoc
        if type(factor) in (float, int):
            self._pos = map(lambda x: x*factor, self._pos)
        elif type(factor) == vector:
            if self._size == factor.size:
                return sum([x[0]*x[1] for x in zip(self._pos, other,pos)])
            else:
                raise ArithmeticError('Unable to multiply vectors of different lengths.')
        else:
            raise NotImplementedError('Unknown factor')
        return self

    def __add__(self, other): #isdoc
        if type(other) == vector:
            if len(self._pos) == len(other.pos):
                self._pos = [x[0]+x[1] for x in zip(self._pos, other.pos)]
            else:
                raise ArithmeticError('Unable to add vectors of different lengths.')
        else:
            raise NotImplementedError('Unknown summand')
        return self

    def __rmul__(self, factor): #isdoc
        return self*factor

    def __len__(self): #isdoc
        if self._size == len(self._pos):
            return self._size
        else:
            raise IndexError('Length cache mismatch.')

    def __div__(self, divisor): #isdoc
        if type(divisor) in (float, int):
            self._pos = map(lambda x: x/divisor, self._pos)
        else:
            raise NotImplementedError('Unknown factor')
        return self

    def random_normalvariate(self, mu = 0, sigma = 1): #isdoc
        self._pos = map(lambda x: random.normalvariate(mu, sigma), self._pos)
        return self

    def random_univariate(self, minimum = 0, maximum = 1): #isdoc
        self._pos = map(lambda x: random.uniform(minimum, maximum), self._pos)
        return self

    def __abs__(self): #isdoc
        return math.sqrt(sum([x*x for x in self._pos]))

    def __getitem__(self, key): #isdoc
        return self._pos[key]

    def normalize(self): #isdoc
        a = abs(self)
        if a == 0:
            raise ArithmeticError('Unable to normalize null vector')
        return self/a

    def derive_random_normal(self): #isdoc
        js = [x for x in range(self._size) if not self._pos[x] == 0]
        if len(js) == 0:
            raise ArithmeticError('Unable to derive from null vector')
        j = random.choice(js)
        b = vector(self._size).random_normalvariate()
        temp = sum([x[0]*x[1] for x in zip(self._pos, b.pos)])
        temp -= self._pos[j]* b.pos[j]
        temp /= self._pos[j]
        b.pos[j] = -temp
        return b

    def derive_random_angle(self, angle): #isdoc
        b = self.derive_random_normal()
        b.normalize()
        factor = abs(self) * math.sqrt((1/math.cos(angle)**2-1))
        b * factor
        return (self+b)

    def rescale_to_length(self, length): #isdoc
        f = length/abs(self)
        return self * f

def weighted_select(items, weights): #isdoc
    s = sum(weights)
    v = random.uniform(0, s)
    s = 0
    for i, w in enumerate(weights):
        s += w
        if v < s:
            return items[i]

def flattenlist(o): #isdoc
    if isinstance(o, list):
        nonlist = [x for x in o if not isinstance(x, list)]
        listelems = [x for x in o if isinstance(x, list)]
        listelems = [flattenlist(x) for x in listelems]
        listelems = sum(listelems, [])
        return nonlist+listelems
    else:
        return [o]

def rotate_vector(v, n, angle): #isdoc
    if isinstance(v, vector):
        v = v._pos
    if isinstance(n, vector):
        n = n._pos
    
    a = math.sqrt(sum([x*x for x in n]))
    n = map(lambda x: x/a, n)

    n = numpy.array(n)
    v = numpy.array(v)

    return list(n*numpy.dot(n, v) + numpy.cross(math.cos(angle)*numpy.cross(n, v), n) + math.sin(angle)*numpy.cross(n, v))

def distance_line_point(point_line, n, point): #isdoc
    n = numpy.array(n)
    a = numpy.array(point_line)
    p = numpy.array(point)
    n /= numpy.linalg.norm(n)
    d = a - p
    d = d - d.dot(n)*n
    return numpy.linalg.norm(d)

def backtracking_assign_unique(options):
    def _run(sel, options):
        if len(sel) == len(options):
            if list(set(sel)) == sel:
                return True
            else:
                return False

        idx = len(sel)
        for v in options[idx][:]:
            if v in sel:
                continue
            sel.append(v)
            options[idx].remove(v)
            if _run(sel, options):
                return True
            sel.pop()
            options[idx].append(v)

    # TODO: test preconditions
    return _run([], options)
